package operation

import (
	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/aaa"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/nfs"
	"shylinux.com/x/icebergs/base/web"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/operation/src/operation/model"
)

type container struct {
	ice.Space
	Table
	order   string `data:"1"`
	fields  string `data:"space,name"`
	foreach string `name:"foreach" help:"遍历" icon:"bi bi-card-list"`
	create  string `name:"create uid* domain module version" role:"leader"`
	remove  string `name:"remove" role:"leader"`
	apply   string `name:"apply" role:"void"`
	rename  string `name:"rename name" role:"void"`
	open    string `name:"open" role:"void"`
}

func (s container) Inputs(m *ice.Message, arg ...string) {
	switch arg[0] {
	case model.UID:
		m.Cmdy(web.SPACE, ice.OPS, web.DREAM, web.SERVER, mdb.NAME, nfs.MODULE, nfs.VERSION)
	default:
		s.Table.Inputs(m, arg...)
	}
}
func (s container) Foreach(m *ice.Message, arg ...string) {
	m.Cmdy(web.SPACE, "20250211-service", "web.chat.dev.service.gateway", m.ActionKey()).Table(func(value ice.Maps) {
		if s.Select(m.Spawn(), web.SPACE, value[web.SPACE]).Length() == 0 {
			s.Insert(m, web.SPACE, value[web.SPACE])
		}
	})
}
func (s container) Create(m *ice.Message, arg ...string) {
	s.Table.Insert(m, kit.Simple(arg, m.OptionSimple(model.CLOUD_UID))...)
}
func (s container) Rename(m *ice.Message, arg ...string) {
	s.Table.Update(m, m.OptionSimple(model.NAME), m.OptionSimple(model.CLOUD_UID, model.UID)...)
}
func (s container) List(m *ice.Message, arg ...string) {
	if len(arg) == 1 {
		s.Select(m, model.CLOUD_UID, arg[0], model.USER_UID, m.Option(model.USER_UID)).PushAction(s.Open, s.Rename)
		kit.If(m.Length() == 0, func() { m.EchoInfoButton("请申请云主机", s.Apply) })
		if m.IsTech() {
			m.Action(s.Foreach, s.Create, s.Apply)
		} else if s.IsLeader(m) {
			m.Action(s.Create, s.Apply)
		} else if m.Length() > 0 {
			m.Action()
		}
	} else {
		s.SelectDetail(m, model.UID, arg[1], model.CLOUD_UID, arg[0], model.USER_UID, m.Option(model.USER_UID)).PushAction(s.Open, s.Rename)
		m.EchoIFrame(web.S(m.Append(web.SPACE)))
	}
	m.Display("").DisplayCSS("")
}
func (s container) Apply(m *ice.Message, arg ...string) {
	s.Transaction(m, func() {
		msg := m.Spawn()
		msg = s.Limit(msg, 1).SelectForUpdate(msg, "(user_uid IS NULL OR user_uid = '') AND (cloud_uid = ? OR cloud_uid IS NULL OR cloud_uid = '')", m.Option(model.CLOUD_UID))
		if msg.Length() == 0 {
			m.Echo("主机不够用啦！请等待管理员添加。")
			return
		}
		s.Update(m, m.OptionSimple(model.CLOUD_UID, model.USER_UID), msg.AppendSimple(model.UID)...)
		m.Cmd(web.SPACE, msg.Append(web.SPACE), aaa.USER, mdb.CREATE, aaa.TECH, m.Option(model.USER_UID), kit.Dict(ice.MSG_USERROLE, aaa.TECH))
	})
}
func (s container) Open(m *ice.Message, arg ...string) {
	m.ProcessOpen(m.Options(ice.MSG_USERPOD, "").MergePod(m.Option(web.SPACE)))
}

func init() { ice.TeamCtxCmd(container{}) }
